# Created by Hugo Cruz Jimenez, August 2011, KAUST
C* #function [Sa,Sv,Sd] = newm(ga,dt,T,dr);
C* %function [maxA,maxV,maxD] = newm(ga,dt,T,dr);
C* %
C* % function for the calculation of the SDOF dynamic response
C* % using Newmark Method for linear systems (Chopra s book page 177)
C* %
C* % Input: ga - ground-acceleration time series (cm/s^2)
C* %        dt - time sampling of the time history
C* %        T  - vector of periods for the spectral response
C* %        dr - damping ratio (default 5%, ie dr = 0.05);
C* %
C* % Banu Sanli, JULY 2006, modified by M.Mai, AUG 2006
C* % --------------------------------------------------

C* if nargin == 3; dr = 0.05; end

      parameter( dt=0.01, dr=0.05, NPT=13000, NPTS=4100, NP=21, NS=20)

      real g1(NPTS), g2(NPTS), g3(NPTS), g4(NPTS)
      real g(NPTS), ga(NPTS), a(NPTS), v(NPTS), d(NPTS)
      real w, m, k, dc, kk
      real T(NP),Sa(NP),hd(NS),maxA,m1(NP),m2(NP),m3(NP),maxc1,maxc2
      character*80 namefile
      data T/ 0.01, 0.02, 0.03, 0.05, 0.075, 0.1, 0.15, 0.2, 0.25, 0.3,
     1        0.4, 0.5, 0.75, 1.0, 1.5, 2.0, 3.0, 4.0, 5.0, 7.5, 10.0/ 

       open(1,file='namefile',status='old')

      do i = 1, NS 
        read(1,*) hd(i)
      enddo

      do i = 1, NPTS
        read(1,*,end=10) g1(i), g2(i), g3(i)
      enddo

  10  itt=i-1

      do 30 mm=1,3
      if (mm.eq.1) then 
        do i = 1, itt-1
          ga(i)=(g1(i+1)-g1(i))/dt
        enddo
      else if (mm.eq.2) then 
        do i = 1, itt-1
          ga(i)=(g2(i+1)-g2(i))/dt
        enddo
      else
        do i = 1, itt-1
          ga(i)=( (g1(i+1)-g1(i))/dt + (g2(i+1)-g2(i))/dt ) / 2.0
        enddo
      endif

C*% Newmark Method
C*% Variables initialization
      do i = 1,itt
        a(i)=0.0
        v(i)=0.0
        d(i)=0.0
      enddo

C* % Parameters defined for the Newmark method
C* % Linear acceleration method
      ganma = 1.0/2.0;
      beta = 1.0/4.0;

      pi=3.1416

C* %%% loop over all periods of interest to assemple
C* %%% response spectra

C* for i = 1:length(T);
      do i=1,NP
  
C* % Properties of the SDOF  

        w = 2.0*pi/T(i) ! ; % Circular frequency (rad/sec)
        m = 1.0 ! ; % mass
        k = m*w**2 ! ; % stiffness
        dc = 2.0 * dr * m * w ! ; % damping coefficient (kN*s/m) (1 N = 1 kg-m/sec2)

 
C* % System starts from rest
C* % Initial calculations
        a(1) = (-m*ga(1) - dc*v(1) - k*d(1)) / m ! 
  
        kk = k + ganma * dc / beta / dt + m / beta / (dt**2)!  % variable k^ (kN/m)
  
        var_a = m / beta / dt + ganma * dc / beta !  % variable a
        var_b = m / (2.0*beta) + dt * dc * (ganma/(2.0*beta) - 1) !  % variable b
  
C* % Calculation for each time step, i
        do j = 1, itt-1  
          dp = -m * (ga(j+1)-ga(j)) + var_a * v(j) + var_b * a(j) !  % kN
          du = dp / kk !  % m
          dv = ganma * du / beta / dt - ganma * v(j) / beta +
     1         dt * a(j) * (1.0 - ganma/(2.0*beta)) !  % m/sec
          da = du / beta / (dt**2) - v(j) / beta / dt - a(j) / (2.*beta) ! % m/sec2

          d(j+1) = d(j) + du
          v(j+1) = v(j) + dv
          a(j+1) = a(j) + da
        enddo 

        maxA=0.0
        do ii=1,itt
          if (abs(a(ii)+ga(ii)).gt.maxA) then
            maxA = abs(a(ii)+ga(ii))
          else
            maxA = maxA
          endif
        enddo


      if (mm.eq.1) then
        m1(i) = maxA
      else if (mm.eq.2) then
        m2(i) = maxA
      else
        m3(i) = maxA
      endif

      enddo

  30  continue

      do j=1,NP
       write(*,'(f7.3, 4f12.6)') T(j), m1(j), m2(j), m3(j), 
     1                           sqrt(m1(j)*m2(j))
      enddo

      end
